var fmdb = require('formidable'),
  redis = require('redis'),
  crypto = require("crypto"),
  moment = require("moment"),
  fs = require('fs');

var host = 'http://localhost:8080/';
var s3Conf;
var client;

var _ex = module.exports;

_ex.configS3 = function(s3Conf) {
  this.s3Conf = s3Conf;
  return this;
};

_ex.s3Policy = function(mimetype, cb) {

  var policy = {
    expiration: moment.utc().add('minutes', 30).format('YYYY-MM-DDTHH:mm:ss\\Z'),
    conditions: [
      { bucket: this.s3Conf.bucket },
      { acl: "public-read" },
      { success_action_redirect: "http://example.com/uploadsuccess" },
      ["starts-with", "$Content-Disposition", ""],
      ["starts-with", "$key", "someFilePrefix_"],
      ["content-length-range", 0, 2147483648],
      ["eq", "$Content-Type", mimetype]
    ]
  };

  var policy64 = new Buffer(JSON.stringify(policy)).toString('base64');

  var cred = {
    signature: crypto.createHmac("sha1", this.s3Conf.secret).update(policy64).digest("base64"),
    policy: policy64,
    key: this.s3Conf.key,
    redirect: this.s3Conf.redirect
  };

  cb(cred);
};

/* Store a the picture and data in a multipart POST in Redis in s3. Can have a callback which return the picture key*/
_ex.storeForm = function(req, res, cb) {

  function redis_store (key, fields) {
    for(var field in fields) {
      client.hset(key, field, fields[field]);
    }
    client.zadd("timeline", Date.now(), key);
  }

  function on_end(err, fields, files) {

    redis_store(key, fields);

    if(cb) cb(res, key);
    else {
      res.end();
    }
  }

  var form = new fmdb.IncomingForm();
  form.uploadDir = img_dir;
  form.parse(req, on_end);
};

/* Serve an image from its key. */
_ex.serve = function(res, key) {
  res.writeHead(200, {
    'content-type': 'image/png'
  });
  res.end(fs.readFileSync('./' + key), 'binary');
};

/* Return the array of the latest X pictures with offset Y. */
_ex.list = function(res, X, Y, cb) {
  client.zrevrangebyscore('timeline', Date.now(), -1, cb);
};

/* Build HTML response of the timeline. For quick sample purpose only*/
_ex.htmlTimeline = function(res) {

  function htmlCB(err, result) {
    res.writeHead(200, {
      'content-type': 'text/html'
    });
    res.write('<table>');
    for(i = 0; i < result.length; i++) {
      res.write('<tr><td><img width="150" src="' + host + result[i] + '"/></td></tr>');
    }
    res.write('</table>');
    res.end();
  }
  _ex.list(res, 0, 10, htmlCB);
};

_ex.redisStart = function(port, host, password, cb) {
  client = redis.createClient(port, host, undefined);
  if(password) {
    client.auth(password, cb);
  } else {
    cb();
  }
};

_ex.redisStop = function() {
  client.end();
};